import { sendMessageLogToSlack } from './notify';
const LEVEL_DEBUG = {
  INFO: 'info',
  WARN: 'warn',
  ERROR: 'error'
};
export const request = my.request;
class Logger {
  isSendToSlack = false;
  isApplyConsoleLog = false;
  slackUrl = '';
  appName = '';
  whitelist = ['*'];
  trackingLog = [];
  trackingRequest = [];
  #getDebug = () => {
    const _getConfig = this.#getConfig.bind(this);
    return {
      log(...args) {
        addLog(args, LEVEL_DEBUG.INFO, _getConfig());
      },
      warn(...args) {
        addLog(args, LEVEL_DEBUG.WARN, _getConfig());
      },
      error(...args) {
        addLog(args, LEVEL_DEBUG.ERROR, _getConfig());
      }
    };
  };
  #getRequest = options => {
    const _trackingRequest = this.trackingRequest;
    let response,
      resHeaders,
      isError = false;
    let {
      success,
      fail,
      complete,
      url,
      headers,
      method,
      data
    } = options;
    request({
      ...options,
      includeHeader: true,
      success: res => {
        success && success(res.data);
        response = res.data;
        resHeaders = res.headers.map ? res.headers.map : res.headers;
      },
      fail: error => {
        fail && fail(error);
        isError = true;
        let cUrlHeader = [];
        for (const key in headers) {
          cUrlHeader.push(`--header '${key}: ${headers[key]}'`);
        }
        const cUrl = `curl --location --request ${method} '${url}' \\\n${cUrlHeader.join(' \\\n')}${data ? ` \\\n--data-raw '${JSON.stringify(data)}'` : ''}
      `;
        my.debug.error(`Fail request API ${url}: ${error}\n${cUrl}`);
        response = error;
      },
      complete(res) {
        complete && complete(res);
        const requestName = url.replace(/^(http:|https:\/\/)?(www\\.)?/i, '').split('/');
        _trackingRequest.push({
          general: {
            url,
            method: method || 'GET',
            time: new Date().toString()
          },
          data,
          headers,
          resHeaders,
          requestInfo: {
            title: requestName.pop(),
            subTitle: requestName.join('/')
          },
          response: JSON.stringify(response, null, 2),
          isError,
          time: new Date()
        });
      }
    });
  };
  debug = this.#getDebug();
  request = this.#getRequest;
  init({
    isOverwriteLog,
    isOverwriteRequest,
    isApplyConsoleLog,
    config
  }) {
    this.#setConfig({
      ...config,
      isApplyConsoleLog: isApplyConsoleLog || false
    });
    if (isOverwriteLog) {
      my.debug = this.debug;
    }
    if (isOverwriteRequest) {
      my.request = this.request;
    }
  }
  #setConfig({
    whitelist,
    isSendToSlack,
    slackUrl,
    appName,
    isApplyConsoleLog
  }) {
    if (whitelist) this.whitelist = whitelist;
    if (isSendToSlack) this.isSendToSlack = isSendToSlack;
    if (slackUrl) this.slackUrl = slackUrl;
    if (appName) this.appName = appName;
    if (isApplyConsoleLog) this.isApplyConsoleLog = isApplyConsoleLog;
  }
  #getConfig() {
    const {
      appName,
      isSendToSlack,
      slackUrl,
      trackingLog,
      isApplyConsoleLog
    } = this;
    return {
      appName,
      isSendToSlack,
      slackUrl,
      trackingLog,
      isApplyConsoleLog
    };
  }
}
export default new Logger();
const stringifyDebug = items => {
  const {
    detail,
    message
  } = items.reduce(({
    detail,
    message
  }, item) => {
    message.push(JSON.stringify(item));
    detail.push(JSON.stringify(item, null, 2));
    return {
      detail,
      message
    };
  }, {
    detail: [],
    message: []
  });
  return {
    message: message.join(', '),
    detail: detail.join('\n\n')
  };
};
function addLog(msgs, type, config) {
  const {
    detail,
    message
  } = stringifyDebug(msgs);
  const time = new Date();
  const {
    trackingLog,
    isSendToSlack,
    appName,
    slackUrl,
    isApplyConsoleLog
  } = config;
  trackingLog.push({
    message,
    detail,
    type,
    time
  });
  if (isSendToSlack && type === LEVEL_DEBUG.ERROR) {
    sendMessageLogToSlack({
      message: msgs,
      type
    }, appName, slackUrl);
  }
  if (isApplyConsoleLog) {
    console.log(...msgs);
  }
}